#include <map>

#include "ttl_define.h"
#include "tt_util.h"
#include "tt_exception.h"

#include "tt_command_line_analyzer.h"


// -- TtCommandLineOption ------------------------------------------------
TtCommandLineOption::TtCommandLineOption( const std::string& main_option,
                                          unsigned int       operand_count,
                                          bool               can_duplicate,
                                          bool               is_deniable,
                                          bool               is_indispensable ) :
main_option_( main_option ),
option_strings_(),
operand_(),
operand_count_( operand_count ),
can_duplicate_( can_duplicate ),
is_deniable_( is_deniable ),
is_indispensable_( is_indispensable ),
is_specified_( false ),
is_denied_( false )
{
  option_strings_.insert( main_option_ );
}

TtCommandLineOption::~TtCommandLineOption()
{
}


const std::string&
TtCommandLineOption::GetMainOption( void ) const
{
  return main_option_;
}


void
TtCommandLineOption::AddOptionString( const std::string& option )
{
  option_strings_.insert( option );
}

const std::set<std::string>&
TtCommandLineOption::GetOptionStrings( void ) const
{
  return option_strings_;
}

void
TtCommandLineOption::SetCanDuplicate( bool flag )
{
  can_duplicate_ = flag;
}

bool
TtCommandLineOption::CanDuplicate( void ) const
{
  return can_duplicate_;
}

void
TtCommandLineOption::SetDeniable( bool flag )
{
  is_deniable_ = flag;
}

bool
TtCommandLineOption::IsDeniable( void ) const
{
  return is_deniable_;
}

void
TtCommandLineOption::SetIndispensable( bool flag )
{
  is_indispensable_ = flag;
}

bool
TtCommandLineOption::IsIndispensable( void ) const
{
  return is_indispensable_;
}

bool
TtCommandLineOption::IsSpecified( void ) const
{
  return is_specified_;
}

bool
TtCommandLineOption::IsDenied( void ) const
{
  return is_denied_;
}

const std::vector<std::string>&
TtCommandLineOption::GetOperand( void ) const
{
  return operand_;
}


int
TtCommandLineOption::ConvertOperandAsInteger( unsigned int index, int min, int max ) const
{
  const std::string& operand = this->GetOperand()[index];
  int tmp;
  if ( NOT( TtUtil::StringToInteger( operand, &tmp, 10 ) ) ) {
    throw TtCommandLineOptionOperandTypeException( main_option_, operand, typeid( int ).name() );
  }
  if ( tmp < min || tmp > max ) {
    throw TtCommandLineOptionOperandRangeException<int>( main_option_, operand, min, max );
  }
  return tmp;
}

double
TtCommandLineOption::ConvertOperandAsDouble( unsigned int index, double min, double max ) const
{
  const std::string& operand = this->GetOperand()[index];
  double tmp;
  if ( NOT( TtUtil::StringToDouble( operand, &tmp ) ) ) {
    throw TtCommandLineOptionOperandTypeException( main_option_, operand, typeid( double ).name() );
  }
  if ( tmp < min || tmp > max ) {
    throw TtCommandLineOptionOperandRangeException<double>( main_option_, operand, min, max );
  }
  return tmp;
}

// -- TtCommandLineOptionBoolean -----------------------------------------
TtCommandLineOptionBoolean::TtCommandLineOptionBoolean( const std::string& option, bool is_indispensable ) :
TtCommandLineOption( option, 0, true, true, is_indispensable )
{
}


bool
TtCommandLineOptionBoolean::GetResult( void ) const
{
  return this->IsSpecified() && NOT( this->IsDenied() );
}


// -- TtCommandLineOptionSingleOperand -----------------------------------
TtCommandLineOptionSingleOperand::TtCommandLineOptionSingleOperand( const std::string& option, bool is_indispensable ) :
TtCommandLineOption( option, 1, false, false, is_indispensable )
{
}

const std::string&
TtCommandLineOptionSingleOperand::GetSingleOperand( void ) const
{
  return this->GetOperand()[0];
}

int
TtCommandLineOptionSingleOperand::ConvertSingleOperandAsInteger( int min, int max ) const
{
  return this->ConvertOperandAsInteger( 0, min, max );
}

double
TtCommandLineOptionSingleOperand::ConvertSingleOperandAsDouble( double min, double max ) const
{
  return this->ConvertOperandAsDouble( 0, min, max );
}

// -- TtCommandLineOptionAnalyzer ----------------------------------------
TtCommandLineOptionAnalyzer::TtCommandLineOptionAnalyzer() :
options_()
{
}

TtCommandLineOptionAnalyzer::~TtCommandLineOptionAnalyzer()
{
}

void
TtCommandLineOptionAnalyzer::Regist( TtCommandLineOption& option )
{
  options_.insert( &option );
}

std::vector<std::string>
TtCommandLineOptionAnalyzer::AnalyzeForConsole( unsigned int argc, const char* const* argv )
{
  std::vector<std::string> args;
  for ( unsigned int i = 0; i < argc; ++i ) {
    args.push_back( argv[i] );
  }
  return this->Analyze( args );
}

std::vector<std::string>
TtCommandLineOptionAnalyzer::AnalyzeForWindows( const std::string& args_string )
{
  std::vector<std::string> args;
  std::string tmp;
  enum {
    DEFAULT,
    IN_DOUBLE_QUOTE,
    SEPARATOR,
  } status = DEFAULT;

  for ( unsigned int i = 0; i < args_string.size(); ++i ) {
    const char c = args_string[i];
    switch ( status ) {
    case DEFAULT:
      switch ( c ) {
      case ' ':
      case '\t':
        status = SEPARATOR;
        args.push_back( tmp );
        tmp.clear();
        break;

      case '\"':
        status = IN_DOUBLE_QUOTE;
        break;

      case '\\':
        if ( i + 1 < args_string.size() && args_string[i + 1] == '\"' ) {
          tmp.append( 1, '\"' );
          i += 1;
        }
        else {
          tmp.append( 1, '\\' );
        }
        break;

      default:
        tmp.append( 1, c );
        break;
      }
      break;

    case IN_DOUBLE_QUOTE:
      switch ( c ) {
      case '\"':
        status = DEFAULT;
        break;

      case '\\':
        if ( i + 1 < args_string.size() && args_string[i + 1] == '\"' ) {
          tmp.append( 1, '\"' );
          i += 1;
        }
        else {
          tmp.append( 1, '\\' );
        }
        break;

      default:
        tmp.append( 1, c );
        break;
      }
      break;

    case SEPARATOR:
      switch ( c ) {
      case ' ':
      case '\t':
        break;

      default:
        --i;
        status = DEFAULT;
        break;
      }
      break;
    }
  }

  if ( NOT( tmp.empty() ) ) {
    args.push_back( tmp );
  }
  return this->Analyze( args );
}


std::vector<std::string>
TtCommandLineOptionAnalyzer::Analyze( const std::vector<std::string> args )
{
  std::map<std::string, TtCommandLineOption*> table;
  for ( std::set<TtCommandLineOption*>::const_iterator option = options_.begin(); option != options_.end(); ++option ) {
    const std::set<std::string>& option_strings = (*option)->GetOptionStrings();
    for ( std::set<std::string>::const_iterator key = option_strings.begin(); key != option_strings.end(); ++key ) {
      if ( table.find( *key ) != table.end() ) {
        throw TtCommandLineOptionDuplicateDefinitionException( *key );
      }
      table[*key] = *option;
      if ( (*option)->IsDeniable() ) {
        std::string deny_key = (*key) + "-";
        if ( table.find( deny_key ) != table.end() ) {
          throw TtCommandLineOptionDuplicateDefinitionException( deny_key );
        }
        table[deny_key] = *option;
      }
    }
  }

  std::vector<std::string> rest;
  for ( std::vector<std::string>::const_iterator it = args.begin(); it != args.end(); ++it ) {
    if ( (*it)[0] == '-' ) {
      std::string option_string = it->substr( 1 );
      if ( table.find( option_string ) == table.end() ) {
        throw TtCommandLineOptionUnknownOptionSpecifiedException( (*it) );
      }
      TtCommandLineOption& option = *table[option_string];

      if ( NOT( option.can_duplicate_ ) && option.is_specified_ ) {
        throw TtCommandLineOptionDuplicatedSpecifyException( option_string );
      }
      option.is_specified_ = true;
      option.is_denied_ = option_string[option_string.size() - 1] == '-';

      option.operand_.clear();
      for ( unsigned int i = 0; i < option.operand_count_; ++i ) {
        ++it;
        if ( it == args.end() || (*it)[0] == '-' ) {
          throw TtCommandLineOptionOperandCountException( option_string );
        }
        option.operand_.push_back( *it );
      }
    }
    else {
      rest.push_back( *it );
    }
  }

  for ( std::map<std::string, TtCommandLineOption*>::iterator it = table.begin(); it != table.end(); ++it ) {
    if ( it->second->is_indispensable_ && NOT( it->second->is_specified_ ) ) {
      throw TtCommandLineOptionIndispensableIsNoneException( it->second->GetMainOption() );
    }
  }
  return rest;
}


// -- TtCommandLineOptionOperandCountException ----------------------------------
TtCommandLineOptionOperandCountException::TtCommandLineOptionOperandCountException( const std::string& option ) :
TtWithCommandLineOptionString( option )
{
}

// -- TtCommandLineOptionDuplicateDefinitionException ---------------------
TtCommandLineOptionDuplicateDefinitionException::TtCommandLineOptionDuplicateDefinitionException( const std::string& option ) :
TtWithCommandLineOptionString( option )
{
}

// -- TtCommandLineOptionIndispensableIsNoneException ---------------------
TtCommandLineOptionIndispensableIsNoneException::TtCommandLineOptionIndispensableIsNoneException( const std::string& option ) :
TtWithCommandLineOptionString( option )
{
}

// -- TtCommandLineOptionUnknownOptionSpecifiedException ------------------
TtCommandLineOptionUnknownOptionSpecifiedException::TtCommandLineOptionUnknownOptionSpecifiedException( const std::string& specified_string ) :
specified_string_( specified_string )
{
}
  
const std::string&
TtCommandLineOptionUnknownOptionSpecifiedException::GetSpecifiedString( void ) const
{
  return specified_string_;
}

// -- TtCommandLineOptionDuplicatedSpecifyException -----------------------
TtCommandLineOptionDuplicatedSpecifyException::TtCommandLineOptionDuplicatedSpecifyException( const std::string& option ) :
TtWithCommandLineOptionString( option )
{
}

// -- TtCommandLineOptionOperandTypeException ----------------------------
TtCommandLineOptionOperandTypeException::TtCommandLineOptionOperandTypeException(
  const std::string& option,
  const std::string& operand,
  const std::string& require_type ) :
TtWithCommandLineOptionString( option ),
TtWithCommandLineOperandString( operand ),
require_type_( require_type )
{
}

const std::string&
TtCommandLineOptionOperandTypeException::GetRequireType( void ) const
{
  return require_type_;
}
