#include "ttl_define.h"
#include "tt_exception.h"

#include "tt_form.h"


const char* const
TtForm::CLASS_NAME = "TTFORM";

std::set<TtForm*>
TtForm::FORM_TABLE;


void
TtForm::Initialize( void )
{
  Initialize( ::GetModuleHandle( NULL ) );
}

void
TtForm::Initialize( HINSTANCE h_instance )
{
  WNDCLASS window_class = TtForm::MakeDefaultWindowClass( h_instance, CLASS_NAME );
  TtForm::RegisterClass( &window_class );
}

void
TtForm::RegisterClass( WNDCLASS* window_class )
{
  if ( ::RegisterClass( window_class ) == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::RegisterClass ) );
  }
}

WNDCLASS
TtForm::MakeDefaultWindowClass( HINSTANCE h_instance, const char* class_name )
{
  WNDCLASS window_class;
  window_class.style         = CS_HREDRAW | CS_VREDRAW;
  window_class.lpfnWndProc   = TtWindow::WindowProcedureForTTL;
  window_class.cbClsExtra    = 0;
  window_class.cbWndExtra    = 0;
  window_class.hInstance     = h_instance;
  window_class.hIcon         = ::LoadIcon( NULL , IDI_APPLICATION );
  window_class.hCursor       = ::LoadCursor( NULL, IDC_ARROW );
  window_class.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
  window_class.lpszMenuName  = NULL;
  window_class.lpszClassName = class_name;
  return window_class;
}

int
TtForm::LoopDispatchMessage( void )
{
  for (;;) {
    MSG msg;
    switch ( ::GetMessage( &msg, NULL, 0, 0 ) ) {
    case -1:
      throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetMessage ) );

    case 0:
      return msg.wParam;

    default:
      if ( TtForm::IsDialogMessage( &msg ) ) {
        break;
      }
      ::TranslateMessage( &msg );
      ::DispatchMessage( &msg );
      break;
    }
  }
}

bool
TtForm::IsDialogMessage( MSG* msg )
{
  for ( std::set<TtForm*>::iterator it = FORM_TABLE.begin(); it != FORM_TABLE.end(); ++it ) {
    if ( (*it)->handle_ != NULL && ::IsDialogMessage( (*it)->handle_, msg ) ) {
      return true;
    }
  }
  return false;
}


DWORD
TtForm::GetStyle( void )
{
  return WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX;
}

DWORD
TtForm::GetExtendedStyle( void )
{
  return WS_EX_STATICEDGE;
}

PCTSTR
TtForm::GetClassName( void )
{
  return CLASS_NAME;
}

bool
TtForm::Created( void )
{
  return true;
}

TtForm::TtForm( void ) :
TtWindow(),
exit_code_( 0 )
{
  FORM_TABLE.insert( this );
}

TtForm::~TtForm()
{
  if ( FORM_TABLE.find( this ) != FORM_TABLE.end() ) {
    FORM_TABLE.erase( this );
  }
}


TtMenu
TtForm::GetSystemMenu( void )
{
  return TtMenu( ::GetSystemMenu( handle_, FALSE ), *this, false );
}



LRESULT
TtForm::WMCloseReceived( void )
{
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMDestroyReceived( void )
{
  PostQuitMessage( exit_code_ );
  return 0;
}

LRESULT
TtForm::WMCommandReceived( int id, int code, HWND child )
{
  NOT_USE_VAR( id );
  NOT_USE_VAR( code );
  NOT_USE_VAR( child );
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMNotifyReceived( int id, int code, HWND child )
{
  NOT_USE_VAR( id );
  NOT_USE_VAR( code );
  NOT_USE_VAR( child );
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMSizeReceived( int flag, int width, int height )
{
  NOT_USE_VAR( flag );
  NOT_USE_VAR( width );
  NOT_USE_VAR( height );
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMPaintReceived( void )
{
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMDropFilesReceived( HDROP drop )
{
  NOT_USE_VAR( drop );
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMHorizontalScrollReceived( int code, int position, HWND handle )
{
  if ( handle != this->GetHandle() ) {
    return TtWindow::FindWindow( handle )->WMHorizontalScrollReceived( code, position, handle );
  }
  return TtWindow::MSG_DEFAULT;
}

LRESULT
TtForm::WMVerticalScrollReceived( int code, int position, HWND handle )
{
  if ( handle != this->GetHandle() ) {
    return TtWindow::FindWindow( handle )->WMVerticalScrollReceived( code, position, handle );
  }
  return TtWindow::MSG_DEFAULT;
}



LRESULT
TtForm::WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param )
{
  LRESULT ret;
  switch ( msg ) {
  case WM_CLOSE:
    ret = this->WMCloseReceived();
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_DESTROY:
    ret = this->WMDestroyReceived();
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_COMMAND:
    ret = this->WMCommandReceived( LOWORD( w_param ), HIWORD( w_param ), reinterpret_cast<HWND>( l_param ) );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_NOTIFY: {
    NMHDR* tmp = reinterpret_cast<NMHDR*>( l_param );
    ret = this->WMNotifyReceived( tmp->idFrom, tmp->code, tmp->hwndFrom );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;
  }

  case WM_SIZE:
    ret = this->WMSizeReceived( w_param, LOWORD( l_param ), HIWORD( l_param ) );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_PAINT:
    ret = this->WMPaintReceived();
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_DROPFILES:
    ret = this->WMDropFilesReceived( reinterpret_cast<HDROP>( w_param ) );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_HSCROLL:
    ret = this->WMHorizontalScrollReceived( LOWORD( w_param ), HIWORD( w_param ), reinterpret_cast<HWND>( l_param ) );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  case WM_VSCROLL:
    ret = this->WMVerticalScrollReceived( LOWORD( w_param ), HIWORD( w_param ), reinterpret_cast<HWND>( l_param ) );
    if ( ret != TtWindow::MSG_DEFAULT ) { return ret; }
    break;

  }
  return this->TtWindow::WindowProcedure( msg, w_param, l_param );
}
