#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_path.h"
#include "tt_util.h"

#include "tt_ini_file.h"


// -- TtIniFile ----------------------------------------------------------
TtIniFile::TtIniFile() :
filename_( TtPath::GetExecutingFilePathCustomExtension( "ini" ) ),
buffer_( DEFAULT_BUFFER_SIZE )
{
}

TtIniFile::TtIniFile( const std::string& filename ) :
filename_( TtPath::IsRelative( filename ) ? TtPath::GetExpandPath( filename ) : filename ),
buffer_( DEFAULT_BUFFER_SIZE )
{
}

TtIniFile::TtIniFile( unsigned int buffer_size ) :
filename_( TtPath::GetExecutingFilePathCustomExtension( "ini" ) ),
buffer_( buffer_size )
{
}

TtIniFile::TtIniFile( const std::string& filename, unsigned int buffer_size ) :
filename_( TtPath::IsRelative( filename ) ? TtPath::GetExpandPath( filename ) : filename ),
buffer_( buffer_size )
{
}

TtIniFile::~TtIniFile()
{
  this->Flush();
}

const std::string&
TtIniFile::GetFileName( void ) const
{
  return filename_;
}

void
TtIniFile::SetFileName( const std::string& filename )
{
  filename_ = filename;
}

TtIniSection
TtIniFile::GetSection( const std::string& name )
{
  return TtIniSection( *this, name, buffer_ );
}

TtIniSection
TtIniFile::operator []( const std::string& name )
{
  return this->GetSection( name );
}


void
TtIniFile::Flush( void )
{
  WritePrivateProfileString( NULL, NULL, NULL, filename_.c_str() );
}


// -- TtIniSection -------------------------------------------------------
TtIniSection::TtIniSection( TtIniFile& ini_file, const std::string& name, TtTemporalyString& buffer ) :
ini_file_( ini_file ),
name_( name ),
buffer_( buffer )
{
}

TtIniSection::TtIniSection( const TtIniSection& obj ) :
ini_file_( obj.ini_file_ ),
name_( obj.name_ ),
buffer_( obj.buffer_ )
{
}


TtIniFile&
TtIniSection::GetIniFile( void ) const
{
  return ini_file_;
}

const std::string&
TtIniSection::GetName( void ) const
{
  return name_;
}


bool
TtIniSection::GetBoolean( const std::string& key )
{
  return this->GetBoolean( key, false );
}

bool
TtIniSection::GetBoolean( const std::string& key, bool default_value )
{
  return this->GetInteger( key, default_value ? 1 : 0 ) ? true : false;
}

void
TtIniSection::SetBoolean( const std::string& key, bool value )
{
  this->SetInteger( key, value ? 1 : 0 );
}


int
TtIniSection::GetInteger( const std::string& key )
{
  return this->GetInteger( key, 0 );
}

int
TtIniSection::GetInteger( const std::string& key, int default_value )
{
  return GetPrivateProfileInt( name_.c_str(), key.c_str(), default_value, ini_file_.GetFileName().c_str() );
}

void
TtIniSection::SetInteger( const std::string& key, int value )
{
  this->SetString( key, TtUtil::ToStringFrom( value ) );
}


double
TtIniSection::GetDouble( const std::string& key )
{
  return this->GetDouble( key, 0.0 );
}

double
TtIniSection::GetDouble( const std::string& key, double default_value )
{
  std::string tmp = this->GetString( key );
  double value;
  return TtUtil::StringToDouble( tmp, &value ) ? value : default_value;
}

void
TtIniSection::SetDouble( const std::string& key, double value )
{
  this->SetString( key, TtUtil::ToStringFrom( value ) );
}


std::string
TtIniSection::GetString( const std::string& key )
{
  return this->GetString( key, "" );
}

std::string
TtIniSection::GetString( const std::string& key, const std::string& default_value )
{
  GetPrivateProfileString(
    name_.c_str(), key.c_str(), default_value.c_str(),
    buffer_.GetPtr(), buffer_.GetSize(), ini_file_.GetFileName().c_str() );
  return buffer_.GetPtr();
}

void
TtIniSection::SetString( const std::string& key, const std::string& value )
{
  ::SetLastError( 0 );
  ::WritePrivateProfileString( name_.c_str(), key.c_str(), value.c_str(), ini_file_.GetFileName().c_str() );
  if ( ::GetLastError() != 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::WritePrivateProfileString ) );
  }
}


void
TtIniSection::Flush( void )
{
  ini_file_.Flush();
}
