#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"

#include "tt_process.h"


DWORD
TtProcess::GetCurrentProcessId( void )
{
  return ::GetCurrentProcessId();
}

// -- TtProcess::Priority ------------------------------------------------
template <>
TtEnumTable<TtProcess::Priority>::TtEnumTable( void ) {
#define REGIST( NAME ) this->Regist( TtProcess::Priority::NAME, #NAME )
  REGIST( NORMAL );
  REGIST( IDLE );
  REGIST( HIGH );
  REGIST( REALTIME );
  REGIST( BELOW_NORMAL );
  REGIST( ABOVE_NORMAL );
#undef REGIST
}

template <>
TtEnumTable<TtProcess::Priority>
TtEnumBase<TtProcess::Priority>::TABLE;


// -- TtProcess::CreateInfo ----------------------------------------------
TtProcess::CreateInfo::CreateInfo( const std::string& filename ) :
filename_( filename ),
arguments_(),
inherit_handles_( false ),
inherit_standard_streams_( false ),
create_new_process_group_( false ),
priority_( Priority::NORMAL ),
use_search_path_( true ),
create_suspended_( false )
{
}


void
TtProcess::CreateInfo::SetFileName( const std::string& filename )
{
  filename_ = filename;
}

const std::string&
TtProcess::CreateInfo::GetFileName( void ) const
{
  return filename_;
}

void
TtProcess::CreateInfo::SetArguments( const std::string& arg )
{
  arguments_ = arg;
}

const std::string&
TtProcess::CreateInfo::GetArguments( void ) const
{
  return arguments_;
}

void
TtProcess::CreateInfo::SetInheritHandles( bool flag )
{
  inherit_handles_ = flag;
}

bool
TtProcess::CreateInfo::GetInheritHandles( void ) const
{
  return inherit_handles_;
}

void
TtProcess::CreateInfo::SetInheritStandardStreams( bool flag )
{
  inherit_standard_streams_ = flag;
}

bool
TtProcess::CreateInfo::GetInheritStandardStreams( void ) const
{
  return inherit_standard_streams_;
}

void
TtProcess::CreateInfo::SetCreateNewProcessGroup( bool flag )
{
  create_new_process_group_ = flag;
}

bool
TtProcess::CreateInfo::GetCreateNewProcessGroup( void ) const
{
  return create_new_process_group_;
}

void
TtProcess::CreateInfo::SetPriority( TtEnum<Priority> priority )
{
  priority_ = priority;
}

TtEnum<TtProcess::Priority>
TtProcess::CreateInfo::GetPriority( void ) const
{
  return priority_;
}

void
TtProcess::CreateInfo::SetUseSearchPath( bool flag )
{
  use_search_path_ = flag;
}

bool
TtProcess::CreateInfo::GetUseSearchPath( void ) const
{
  return use_search_path_;
}

void
TtProcess::CreateInfo::SetCreateSuspended( bool flag )
{
  create_suspended_ = flag;
}

bool
TtProcess::CreateInfo::GetCreateSuspended( void ) const
{
  return create_suspended_;
}


// -- TtProcess --------------------------------------------------------
TtProcess::TtProcess( void ) :
process_handle_( INVALID_HANDLE_VALUE ),
process_id_( 0 ),
thread_handle_( INVALID_HANDLE_VALUE ),
thread_id_( 0 )
{
}

TtProcess::~TtProcess()
{
  if ( process_handle_ != INVALID_HANDLE_VALUE ) {
    ::CloseHandle( process_handle_ );
    ::CloseHandle( thread_handle_ );
  }
}

void
TtProcess::Create( CreateInfo& info )
{
  SECURITY_ATTRIBUTES sa = {0};
  sa.nLength = sizeof( sa );

  STARTUPINFO startup_info = {};
  startup_info.cb = sizeof( startup_info );
  startup_info.wShowWindow = SW_SHOWNORMAL;
  if ( info.inherit_standard_streams_ ) {
    startup_info.dwFlags |= STARTF_USESTDHANDLES;
    startup_info.hStdInput  = ::GetStdHandle( STD_INPUT_HANDLE );
    startup_info.hStdOutput = ::GetStdHandle( STD_OUTPUT_HANDLE );
    startup_info.hStdError  = ::GetStdHandle( STD_ERROR_HANDLE );
  }

  PROCESS_INFORMATION proc = {INVALID_HANDLE_VALUE};

  TtTemporalyString tmp_args( "\"" + info.filename_ + "\" " + info.arguments_ );

  int ret = CreateProcess(
    info.use_search_path_ ? NULL : info.filename_.c_str(),
    tmp_args.GetPtr(),
    NULL,
    NULL,
    info.inherit_handles_ ? TRUE : FALSE,
    ( info.priority_.ToValue() |
      (info.create_new_process_group_ ? CREATE_NEW_PROCESS_GROUP : 0) |
      (info.create_suspended_ ? CREATE_SUSPENDED : 0 ) ),
    NULL,
    NULL, // current directory
    &startup_info,
    &proc );
  if ( ret == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::CreateProcess ) );
  }
  process_handle_ = proc.hProcess;
  thread_handle_  = proc.hThread;
  process_id_     = proc.dwProcessId;
  thread_id_      = proc.dwThreadId;
}

void
TtProcess::Start( void )
{
  if ( ::ResumeThread( thread_handle_ ) == static_cast<DWORD>( -1 ) ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::ResumeThread ) );
  }
}


void
TtProcess::Wait( void )
{
  if ( process_handle_ == INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  if ( ::WaitForSingleObject( process_handle_, INFINITE ) != WAIT_OBJECT_0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::WaitForSingleObject ) );
  }

}
void
TtProcess::Terminate( UINT exit_code )
{
  if ( ::TerminateProcess( process_handle_, exit_code ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::TerminateProcess ) );
  }

}

bool
TtProcess::HasExited( void ) const
{
  if ( process_handle_ == INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  DWORD ret;
  if ( ::GetExitCodeProcess( process_handle_, &ret ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetExitCodeProcess ) );
  }
  return ret != STILL_ACTIVE;
}


DWORD
TtProcess::GetExitCode( void ) const
{
  if ( process_handle_ == INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  DWORD ret;
  if ( ::GetExitCodeProcess( process_handle_, &ret ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( ::GetExitCodeProcess ) );
  }
  if ( ret == STILL_ACTIVE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  return ret;
}

DWORD
TtProcess::GetProcessId( void ) const
{
  if ( process_handle_ == INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  return process_id_;
}
