#include "ttl_define.h"
#include "tt_squirrel_exception.h"

#include "tt_squirrel.h"

using namespace TtSquirrel;


// -- VirtualMachine::Stack ----------------------------------------------
VirtualMachine::Stack::Stack( HSQUIRRELVM& vm ) :
vm_( vm )
{
}

void
VirtualMachine::Stack::PushRootTable( void )
{
  sq_pushroottable( vm_ );
}

void
VirtualMachine::Stack::Push( int n )
{
  sq_pushinteger( vm_, n );
}

void
VirtualMachine::Stack::Push( float f )
{
  sq_pushfloat( vm_, f );
}

void
VirtualMachine::Stack::Push( const char* str )
{
  sq_pushstring( vm_, str, -1 );
}

void
VirtualMachine::Stack::Push( const std::string& str )
{
  sq_pushstring( vm_, str.c_str(), str.size() );
}

void
VirtualMachine::Stack::Push( bool flag )
{
  sq_pushbool( vm_, flag ? SQTrue : SQFalse );
}

void
VirtualMachine::Stack::Push( void* ptr )
{
  sq_pushuserpointer( vm_, ptr );
}

void
VirtualMachine::Stack::PushNull( void )
{
  sq_pushnull( vm_ );
}


ObjectType
VirtualMachine::Stack::GetType( int index )
{
  return sq_gettype( vm_, index );
}


int
VirtualMachine::Stack::GetAsInteger( int index )
{
  int tmp;
  ObjectTypeCheck( this->GetType( index ), ObjectType::Integer );
  ResultTest( sq_getinteger( vm_, index, &tmp ), FUNC_NAME_OF( sq_getinteger ) );
  return tmp;
}

float
VirtualMachine::Stack::GetAsFloat( int index )
{
  float tmp;
  ObjectTypeCheck( this->GetType( index ), ObjectType::Float );
  ResultTest( sq_getfloat( vm_, index, &tmp ), FUNC_NAME_OF( sq_getfloat ) );
  return tmp;
  
}

std::string
VirtualMachine::Stack::GetAsString( int index )
{
  const char* tmp;
  ObjectTypeCheck( this->GetType( index ), ObjectType::String );
  ResultTest( sq_getstring( vm_, index, &tmp ), FUNC_NAME_OF( sq_getstring ) );
  return tmp;
}

bool
VirtualMachine::Stack::GetAsBoolean( int index )
{
  SQBool tmp;
  ObjectTypeCheck( this->GetType( index ), ObjectType::Boolean );
  ResultTest( sq_getbool( vm_, index, &tmp ), FUNC_NAME_OF( sq_getbool ) );
  return tmp == SQTrue;
}

void*
VirtualMachine::Stack::GetAsUserPointer( int index )
{
  void* tmp;
  ObjectTypeCheck( this->GetType( index ), ObjectType::UserPointer );
  ResultTest( sq_getuserpointer( vm_, index, &tmp ), FUNC_NAME_OF( sq_getuserpointer ) );
  return tmp;
}


void
VirtualMachine::Stack::PopTop( void )
{
  sq_poptop( vm_ );
}


void
VirtualMachine::Stack::Pop( int count )
{
  sq_pop( vm_, count );
}


void
VirtualMachine::Stack::MoveToTop( int index )
{
  sq_push( vm_, index );
}

void
VirtualMachine::Stack::Remove( int index )
{
  sq_remove( vm_, index );
}

void
VirtualMachine::Stack::Get( int index )
{
  ResultTest( sq_get( vm_, index ), FUNC_NAME_OF( sq_get ) );
}

void
VirtualMachine::Stack::Set( int index )
{
  ResultTest( sq_set( vm_, index ), FUNC_NAME_OF( sq_set ) );
}

int
VirtualMachine::Stack::GetTopIndex( void )
{
  return sq_gettop( vm_ );
}


void
VirtualMachine::Stack::Call( int param_count, bool need_return_value , bool raise_exception_in_error )
{
  ResultTest( sq_call( vm_, param_count,
                       need_return_value ? SQTrue : SQFalse,
                       raise_exception_in_error ? SQTrue : SQFalse ),
              FUNC_NAME_OF( sq_call ) );
}


// -- VirtualMachine -----------------------------------------------------
VirtualMachine::VirtualMachine( int stack_size ) :
vm_( sq_open( stack_size ) )
{
  sq_setprintfunc( vm_, TtSquirrel::PrintFunc );
}

VirtualMachine::~VirtualMachine()
{
  sq_close( vm_ );
}

HSQUIRRELVM
VirtualMachine::GetHandle( void )
{
  return vm_;
}


VirtualMachine::Stack
VirtualMachine::GetStack( void )
{
  return Stack( vm_ );
}



void
TtSquirrel::ResultTest( SQRESULT result, const char* function_name )
{
  if ( SQ_FAILED( result ) ) {
    throw SquirrelAPIException( function_name );
  }
}

void
TtSquirrel::ObjectTypeCheck( ObjectType require, ObjectType actual )
{
  if ( require != actual ) {
    throw ObjectTypeException( require, actual );
  }
}

#include <stdarg.h>
#include <stdio.h>

void
TtSquirrel::PrintFunc( HSQUIRRELVM vm, const SQChar* s, ... )
{
  static_cast<void>( vm );

  va_list arglist;
  va_start( arglist, s );
  vprintf( s, arglist );
  va_end( arglist );
}
