#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_temporaly_array.h"

#include "tt_treeview.h"


// -- TtTreeView ---------------------------------------------------------
TtTreeView::TtTreeView() :
root_( *this, TreeView_GetRoot( handle_ ) )
{
  if ( root_.handle_ == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetRoot ) );
  }
}

TtTreeItem&
TtTreeView::GetRoot( void )
{
  return root_;
}

TtTreeItem
TtTreeView::GetSelected( void )
{
  HTREEITEM tmp = TreeView_GetSelection( handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetSelection ) );
  }
  return TtTreeItem( *this, tmp );
}

TtTreeItem
TtTreeView::GetDropHilight( void )
{
  HTREEITEM tmp = TreeView_GetDropHilight( handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetSelection ) );
  }
  return TtTreeItem( *this, tmp );
}

int
TtTreeView::GetCount( void )
{
  return TreeView_GetCount( handle_ );
}

int
TtTreeView::GetVisibleCount( void )
{
  return TreeView_GetVisibleCount( handle_ );
}


void
TtTreeView::SetImageList( TtImageList& image_list )
{
  HIMAGELIST tmp = TreeView_SetImageList( handle_, image_list.GetHandle(), TVSIL_NORMAL );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SetImageList ) );
  }
}

void
TtTreeView::SetImageListState( TtImageList& image_list )
{
  HIMAGELIST tmp = TreeView_SetImageList( handle_, image_list.GetHandle(), TVSIL_STATE );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SetImageList ) );
  }
}


// -- TtTreeItem ---------------------------------------------------------
TtTreeItem::TtTreeItem( TtTreeView& tree, HTREEITEM handle ) :
handle_( handle ),
tree_( tree )
{
}


void
TtTreeItem::GetInfo( TVITEM& info )
{
  TreeView_GetItem( tree_.GetHandle(), &info );
}

void
TtTreeItem::SetInfo( TVITEM& info )
{
  if ( TreeView_SetItem( tree_.GetHandle(), &info ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SetItem ) );
  }
}


HTREEITEM
TtTreeItem::GetHandle( void )
{
  return handle_;
}


TtTreeItem
TtTreeItem::GetParent( void )
{
  HTREEITEM tmp = TreeView_GetParent( tree_.GetHandle(), handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetParent ) );
  }
  return TtTreeItem( tree_, tmp );
}

TtTreeItem
TtTreeItem::GetFirstChild( void )
{
  HTREEITEM tmp = TreeView_GetChild( tree_.GetHandle(), handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetChild ) );
  }
  return TtTreeItem( tree_, tmp );
}

TtTreeItem
TtTreeItem::GetNextBrother( void )
{
  HTREEITEM tmp = TreeView_GetNextSibling( tree_.GetHandle(), handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetNextSibling ) );
  }
  return TtTreeItem( tree_, tmp );
}

TtTreeItem
TtTreeItem::GetPrevBrother( void )
{
  HTREEITEM tmp = TreeView_GetPrevSibling( tree_.GetHandle(), handle_ );
  if ( tmp == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_GetPrevSibling ) );
  }
  return TtTreeItem( tree_, tmp );
}


TtTreeItem
TtTreeItem::AddItem( HTREEITEM prev, const std::string& text )
{
  TVINSERTSTRUCT tmp_insert = {0};
  tmp_insert.hParent = handle_;
  tmp_insert.hInsertAfter = prev;
  if ( NOT( text.empty() ) ) {
    tmp_insert.item.mask = TVIF_TEXT;
    tmp_insert.item.pszText = const_cast<char*>( text.c_str() );
  }
  HTREEITEM tmp_item = TreeView_InsertItem( tree_.GetHandle(), &tmp_insert );
  if ( tmp_item == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_InsertItem ) );
  }
  return TtTreeItem( tree_, tmp_item );
}

TtTreeItem
TtTreeItem::AddItemAfter( TtTreeItem& prev, const std::string& text )
{
  return this->AddItem( prev.handle_, text );
}

TtTreeItem
TtTreeItem::AddItemFirst( const std::string& text )
{
  return this->AddItem( TVI_FIRST, text );
}

TtTreeItem
TtTreeItem::AddItemLast( const std::string& text )
{
  return this->AddItem( TVI_LAST, text );
}

TtTreeItem
TtTreeItem::AddItemSorted( const std::string& text )
{
  return this->AddItem( TVI_SORT, text );
}


void
TtTreeItem::Remove( void )
{
  if ( TreeView_DeleteItem( tree_.GetHandle(), handle_ ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_DeleteItem ) );
  }
}


void
TtTreeItem::SortChildren( void )
{
  if ( TreeView_SortChildren( tree_.GetHandle(), handle_, FALSE ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SortChildren ) );
  }
}

void
TtTreeItem::SortChildrenRecursive( void )
{
  if ( TreeView_SortChildren( tree_.GetHandle(), handle_, TRUE ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SortChildren ) );
  }
}


std::string
TtTreeItem::GetText( void )
{
  TVITEM tmp = {TVIF_TEXT};
  TtTemporalyString buf( 1024 * 16 );
  tmp.pszText = buf.GetPtr();
  tmp.cchTextMax = buf.GetSize();
  this->GetInfo( tmp );
  return std::string( tmp.pszText );
}

void
TtTreeItem::SetText( const std::string& str )
{
  TVITEM tmp = {TVIF_TEXT};
  tmp.pszText = const_cast<char*>( str.c_str() );
  this->SetInfo( tmp );
}


void
TtTreeItem::Expand( void )
{
  TreeView_Expand( tree_.GetHandle(), handle_, TVE_EXPAND );
}

void
TtTreeItem::Collapse( void )
{
  TreeView_Expand( tree_.GetHandle(), handle_, TVE_COLLAPSE );
}

void
TtTreeItem::ToggleExpand( void )
{
  TreeView_Expand( tree_.GetHandle(), handle_, TVE_TOGGLE );
}


void
TtTreeItem::SetEditMode( void )
{
  if ( TreeView_EditLabel( tree_.GetHandle(), handle_ ) == NULL ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_EditLabel ) );
  }
}

void
TtTreeItem::SetSelect( void )
{
  if ( TreeView_SelectItem( tree_.GetHandle(), handle_ ) == FALSE ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TreeView_SelectItem ) );
  }
}


void
TtTreeItem::SetImageIndex( int index )
{
  TVITEM tmp = {TVIF_IMAGE};
  tmp.iImage = index;
  this->SetInfo( tmp );
}

void
TtTreeItem::SetSelectedImageIndex( int index )
{
  TVITEM tmp = {TVIF_SELECTEDIMAGE};
  tmp.iSelectedImage = index;
  this->SetInfo( tmp );
}

void
TtTreeItem::SetBothImageIndex( int index )
{
  TVITEM tmp = {TVIF_IMAGE | TVIF_SELECTEDIMAGE};
  tmp.iImage = index;
  tmp.iSelectedImage = index;
  this->SetInfo( tmp );
}
